@@ -4,9 +4,13 @@
 #![deny(missing_docs)]
 // Increase recursion limit for the `quick_error!` macro.
 #![recursion_limit = "256"]
+// Enable nightly benchmark functionality if "_benchmarks" feature is enabled.
+#![cfg_attr(all(test, feature = "_benchmarks"), feature(test))]
+#[cfg(all(test, feature = "_benchmarks"))]
+extern crate test;
 
 pub use self::decoder::{DecodingError, LoopCount, WebPDecoder};
-pub use self::encoder::{ColorType, EncodingError, WebPEncoder};
+pub use self::encoder::{ColorType, EncoderParams, EncodingError, WebPEncoder};
 
 mod decoder;
 mod encoder;
