@@ -1,5 +1,6 @@
 #![no_std]
 #![allow(non_camel_case_types)]
+#![warn(clippy::doc_markdown)]
 #![warn(clippy::missing_inline_in_public_items)]
 #![allow(clippy::eq_op)]
 #![allow(clippy::excessive_precision)]
@@ -45,6 +46,9 @@ use safe_arch::*;
 
 use bytemuck::*;
 
+#[cfg(feature = "serde")]
+use serde::{ser::SerializeSeq, Deserialize, Serialize};
+
 #[macro_use]
 mod macros;
 
@@ -329,7 +333,7 @@ where
   n ^ ((n ^ y) & mask)
 }
 
-/// given `type.op(type)` and type is Copy, impls `type.op(&type)`
+/// given `type.op(type)` and type is `Copy`, impls `type.op(&type)`
 macro_rules! bulk_impl_op_ref_self_for {
   ($(($op:ident, $method:ident) => [$($t:ty),+]),+ $(,)?) => {
     $( // do each trait/list matching given
@@ -974,3 +978,54 @@ bulk_impl_const_rhs_op!((CmpGt, cmp_gt) => [(f64x4, f64), (f64x2, f64), (f32x4,f
 bulk_impl_const_rhs_op!((CmpNe, cmp_ne) => [(f64x4, f64), (f64x2, f64), (f32x4,f32), (f32x8,f32),]);
 bulk_impl_const_rhs_op!((CmpLe, cmp_le) => [(f64x4, f64), (f64x2, f64), (f32x4,f32), (f32x8,f32),]);
 bulk_impl_const_rhs_op!((CmpGe, cmp_ge) => [(f64x4, f64), (f64x2, f64), (f32x4,f32), (f32x8,f32),]);
+
+macro_rules! impl_serde {
+  ($i:ident, $t:ty) => {
+    #[cfg(feature = "serde")]
+    impl Serialize for $i {
+      #[inline]
+      fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+      where
+        S: serde::Serializer,
+      {
+        let array = self.as_array_ref();
+        let mut seq = serializer.serialize_seq(Some(array.len()))?;
+        for e in array {
+          seq.serialize_element(e)?;
+        }
+        seq.end()
+      }
+    }
+
+    #[cfg(feature = "serde")]
+    impl<'de> Deserialize<'de> for $i {
+      #[inline]
+      fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
+      where
+        D: serde::Deserializer<'de>,
+      {
+        Ok(<$t>::deserialize(deserializer)?.into())
+      }
+    }
+  };
+}
+
+impl_serde!(f32x8, [f32; 8]);
+impl_serde!(f32x4, [f32; 4]);
+impl_serde!(f64x4, [f64; 4]);
+impl_serde!(f64x2, [f64; 2]);
+impl_serde!(i8x16, [i8; 16]);
+impl_serde!(i16x16, [i16; 16]);
+impl_serde!(i8x32, [i8; 32]);
+impl_serde!(i16x8, [i16; 8]);
+impl_serde!(i32x4, [i32; 4]);
+impl_serde!(i32x8, [i32; 8]);
+impl_serde!(i64x2, [i64; 2]);
+impl_serde!(i64x4, [i64; 4]);
+impl_serde!(u8x16, [u8; 16]);
+impl_serde!(u16x8, [u16; 8]);
+impl_serde!(u16x16, [u16; 16]);
+impl_serde!(u32x4, [u32; 4]);
+impl_serde!(u32x8, [u32; 8]);
+impl_serde!(u64x2, [u64; 2]);
+impl_serde!(u64x4, [u64; 4]);
